import 'firebase/auth';
import 'firebase/database';
import { User as FirebaseUser, app as firebaseApp, auth as firebaseAuth } from 'firebase/app';
export declare enum ActionType {
    AuthApplyActionCode = 0,
    AuthCheckActionCode = 1,
    AuthConfirmPasswordReset = 2,
    AuthCreateUserWithEmailAndPassword = 3,
    AuthSendPasswordResetEmail = 4,
    AuthSetPersistence = 5,
    AuthSignInAndRetrieveDataWithCredential = 6,
    AuthSignInAnonymously = 7,
    AuthSignInWithCredential = 8,
    AuthSignInWithCustomToken = 9,
    AuthSignInWithEmailAndPassword = 10,
    AuthSignInWithPhoneNumber = 11,
    AuthSignInWithPopup = 12,
    AuthSignInWithRedirect = 13,
    AuthSignOut = 14,
    AuthUseDeviceLanguage = 15,
    AuthVerifyPasswordResetCode = 16,
    DatabaseGoOffline = 17,
    DatabaseGoOnline = 18,
    ReferencePush = 19,
    ReferenceRemove = 20,
    ReferenceSet = 21,
    ReferenceSetPriority = 22,
    ReferenceSetWithPriority = 23,
    ReferenceTransaction = 24,
    ReferenceUpdate = 25,
    UserUnlink = 26,
    UserUpdateEmail = 27,
    UserUpdatePassword = 28,
    UserUpdatePhoneNumber = 29,
    UserUpdateProfile = 30,
}
export interface FirebaseAction {
    actionType: ActionType;
    as: (category: string) => FirebaseAction;
    [propName: string]: any;
}
export interface UserProfile {
    displayName: null | string;
    photoURL: null | string;
}
export declare type ActionHandler = ((action: FirebaseAction) => Promise<any>);
export declare type UpdateFn = ((value: any) => any);
export declare const firebaseActions: {
    auth: {
        applyActionCode: (code: string) => FirebaseAction;
        checkActionCode: (code: string) => FirebaseAction;
        confirmPasswordReset: (code: string, newPassword: string) => FirebaseAction;
        createUserWithEmailAndPassword: (email: string, password: string) => FirebaseAction;
        sendPasswordResetEmail: (email: string) => FirebaseAction;
        setPersistence: (persistence: string) => FirebaseAction;
        signInAndRetrieveDataWithCredential: (credential: firebaseAuth.AuthCredential) => FirebaseAction;
        signInAnonymously: () => FirebaseAction;
        signInWithCredential: (credential: firebaseAuth.AuthCredential) => FirebaseAction;
        signInWithCustomToken: (token: string) => FirebaseAction;
        signInWithEmailAndPassword: (email: string, password: string) => FirebaseAction;
        signInWithPhoneNumber: (phoneNumber: string, verifier: firebaseAuth.ApplicationVerifier) => FirebaseAction;
        signInWithPopup: (provider: firebaseAuth.AuthProvider) => FirebaseAction;
        signInWithRedirect: (provider: firebaseAuth.AuthProvider) => FirebaseAction;
        signOut: () => FirebaseAction;
        useDeviceLanguage: () => FirebaseAction;
        user: (user: FirebaseUser) => {
            unlink: () => FirebaseAction;
            updateEmail: (email: string) => FirebaseAction;
            updatePassword: (password: string) => FirebaseAction;
            updatePhoneNumber: (phoneNumber: string) => FirebaseAction;
            updateProfile: (profile: UserProfile) => FirebaseAction;
        };
        verifyPasswordResetCode: (code: string) => FirebaseAction;
    };
    database: {
        goOffline: () => FirebaseAction;
        goOnline: () => FirebaseAction;
        ref: (refPath: string) => {
            push: (value: any) => FirebaseAction;
            remove: () => FirebaseAction;
            set: (value: any) => FirebaseAction;
            setPriority: (priority: string | number | null) => FirebaseAction;
            setWithPriority: (value: any, priority: string | number | null) => FirebaseAction;
            transaction: (updateFn: (value: any) => any) => FirebaseAction;
            update: (values: any) => FirebaseAction;
        };
    };
};
export declare function makeActionHandler(app: firebaseApp.App): ActionHandler;
