"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("firebase/auth");
require("firebase/database");
var ActionType;
(function (ActionType) {
    ActionType[ActionType["AuthApplyActionCode"] = 0] = "AuthApplyActionCode";
    ActionType[ActionType["AuthCheckActionCode"] = 1] = "AuthCheckActionCode";
    ActionType[ActionType["AuthConfirmPasswordReset"] = 2] = "AuthConfirmPasswordReset";
    ActionType[ActionType["AuthCreateUserWithEmailAndPassword"] = 3] = "AuthCreateUserWithEmailAndPassword";
    ActionType[ActionType["AuthSendPasswordResetEmail"] = 4] = "AuthSendPasswordResetEmail";
    ActionType[ActionType["AuthSetPersistence"] = 5] = "AuthSetPersistence";
    ActionType[ActionType["AuthSignInAndRetrieveDataWithCredential"] = 6] = "AuthSignInAndRetrieveDataWithCredential";
    ActionType[ActionType["AuthSignInAnonymously"] = 7] = "AuthSignInAnonymously";
    ActionType[ActionType["AuthSignInWithCredential"] = 8] = "AuthSignInWithCredential";
    ActionType[ActionType["AuthSignInWithCustomToken"] = 9] = "AuthSignInWithCustomToken";
    ActionType[ActionType["AuthSignInWithEmailAndPassword"] = 10] = "AuthSignInWithEmailAndPassword";
    ActionType[ActionType["AuthSignInWithPhoneNumber"] = 11] = "AuthSignInWithPhoneNumber";
    ActionType[ActionType["AuthSignInWithPopup"] = 12] = "AuthSignInWithPopup";
    ActionType[ActionType["AuthSignInWithRedirect"] = 13] = "AuthSignInWithRedirect";
    ActionType[ActionType["AuthSignOut"] = 14] = "AuthSignOut";
    ActionType[ActionType["AuthUseDeviceLanguage"] = 15] = "AuthUseDeviceLanguage";
    ActionType[ActionType["AuthVerifyPasswordResetCode"] = 16] = "AuthVerifyPasswordResetCode";
    ActionType[ActionType["DatabaseGoOffline"] = 17] = "DatabaseGoOffline";
    ActionType[ActionType["DatabaseGoOnline"] = 18] = "DatabaseGoOnline";
    ActionType[ActionType["ReferencePush"] = 19] = "ReferencePush";
    ActionType[ActionType["ReferenceRemove"] = 20] = "ReferenceRemove";
    ActionType[ActionType["ReferenceSet"] = 21] = "ReferenceSet";
    ActionType[ActionType["ReferenceSetPriority"] = 22] = "ReferenceSetPriority";
    ActionType[ActionType["ReferenceSetWithPriority"] = 23] = "ReferenceSetWithPriority";
    ActionType[ActionType["ReferenceTransaction"] = 24] = "ReferenceTransaction";
    ActionType[ActionType["ReferenceUpdate"] = 25] = "ReferenceUpdate";
    ActionType[ActionType["UserUnlink"] = 26] = "UserUnlink";
    ActionType[ActionType["UserUpdateEmail"] = 27] = "UserUpdateEmail";
    ActionType[ActionType["UserUpdatePassword"] = 28] = "UserUpdatePassword";
    ActionType[ActionType["UserUpdatePhoneNumber"] = 29] = "UserUpdatePhoneNumber";
    ActionType[ActionType["UserUpdateProfile"] = 30] = "UserUpdateProfile";
})(ActionType = exports.ActionType || (exports.ActionType = {}));
function action(actionType, props) {
    if (props === void 0) { props = {}; }
    return Object.assign({
        actionType: actionType,
        as: function (category) {
            return action(actionType, Object.assign({ category: category }, props));
        }
    }, props);
}
exports.firebaseActions = {
    auth: {
        applyActionCode: function (code) {
            return action(ActionType.AuthApplyActionCode, { code: code });
        },
        checkActionCode: function (code) {
            return action(ActionType.AuthCheckActionCode, { code: code });
        },
        confirmPasswordReset: function (code, newPassword) {
            return action(ActionType.AuthConfirmPasswordReset, { code: code, newPassword: newPassword });
        },
        createUserWithEmailAndPassword: function (email, password) {
            return action(ActionType.AuthCreateUserWithEmailAndPassword, {
                email: email,
                password: password
            });
        },
        sendPasswordResetEmail: function (email) {
            return action(ActionType.AuthSendPasswordResetEmail, { email: email });
        },
        setPersistence: function (persistence) {
            return action(ActionType.AuthSetPersistence, { persistence: persistence });
        },
        signInAndRetrieveDataWithCredential: function (credential) {
            return action(ActionType.AuthSignInAndRetrieveDataWithCredential, {
                credential: credential
            });
        },
        signInAnonymously: function () { return action(ActionType.AuthSignInAnonymously); },
        signInWithCredential: function (credential) {
            return action(ActionType.AuthSignInWithCredential, { credential: credential });
        },
        signInWithCustomToken: function (token) {
            return action(ActionType.AuthSignInWithCustomToken, { token: token });
        },
        signInWithEmailAndPassword: function (email, password) {
            return action(ActionType.AuthSignInWithEmailAndPassword, { email: email, password: password });
        },
        signInWithPhoneNumber: function (phoneNumber, verifier) {
            return action(ActionType.AuthSignInWithPhoneNumber, { phoneNumber: phoneNumber, verifier: verifier });
        },
        signInWithPopup: function (provider) {
            return action(ActionType.AuthSignInWithPopup, { provider: provider });
        },
        signInWithRedirect: function (provider) {
            return action(ActionType.AuthSignInWithRedirect, { provider: provider });
        },
        signOut: function () { return action(ActionType.AuthSignOut); },
        useDeviceLanguage: function () { return action(ActionType.AuthUseDeviceLanguage); },
        user: function (user) { return ({
            unlink: function () { return action(ActionType.UserUnlink, { user: user }); },
            updateEmail: function (email) {
                return action(ActionType.UserUpdateEmail, { email: email, user: user });
            },
            updatePassword: function (password) {
                return action(ActionType.UserUpdatePassword, { password: password, user: user });
            },
            updatePhoneNumber: function (phoneNumber) {
                return action(ActionType.UserUpdatePhoneNumber, { phoneNumber: phoneNumber, user: user });
            },
            updateProfile: function (profile) {
                return action(ActionType.UserUpdateProfile, { profile: profile, user: user });
            }
        }); },
        verifyPasswordResetCode: function (code) {
            return action(ActionType.AuthVerifyPasswordResetCode, { code: code });
        }
    },
    database: {
        goOffline: function () { return action(ActionType.DatabaseGoOffline); },
        goOnline: function () { return action(ActionType.DatabaseGoOnline); },
        ref: function (refPath) { return ({
            push: function (value) {
                return action(ActionType.ReferencePush, { refPath: refPath, value: value });
            },
            remove: function () { return action(ActionType.ReferenceRemove, { refPath: refPath }); },
            set: function (value) { return action(ActionType.ReferenceSet, { refPath: refPath, value: value }); },
            setPriority: function (priority) {
                return action(ActionType.ReferenceSetPriority, { priority: priority, refPath: refPath });
            },
            setWithPriority: function (value, priority) {
                return action(ActionType.ReferenceSetWithPriority, {
                    priority: priority,
                    refPath: refPath,
                    value: value
                });
            },
            transaction: function (updateFn) {
                return action(ActionType.ReferenceTransaction, { refPath: refPath, updateFn: updateFn });
            },
            update: function (values) {
                return action(ActionType.ReferenceUpdate, { refPath: refPath, values: values });
            }
        }); }
    }
};
function makeActionHandler(app) {
    var auth = app.auth();
    var db = app.database();
    function handleAction(action) {
        switch (action.actionType) {
            case ActionType.AuthApplyActionCode:
                return auth.applyActionCode(action.code);
            case ActionType.AuthCheckActionCode:
                return auth.checkActionCode(action.code);
            case ActionType.AuthConfirmPasswordReset:
                return auth.confirmPasswordReset(action.code, action.newPassword);
            case ActionType.AuthCreateUserWithEmailAndPassword:
                return auth.createUserWithEmailAndPassword(action.email, action.password);
            case ActionType.AuthSendPasswordResetEmail:
                return auth.sendPasswordResetEmail(action.email);
            case ActionType.AuthSetPersistence:
                return auth.setPersistence(action.persistence);
            case ActionType.AuthSignInAndRetrieveDataWithCredential:
                return auth.signInAndRetrieveDataWithCredential(action.credential);
            case ActionType.AuthSignInAnonymously:
                return auth.signInAnonymously();
            case ActionType.AuthSignInWithCredential:
                return auth.signInWithCredential(action.credential);
            case ActionType.AuthSignInWithCustomToken:
                return auth.signInWithCustomToken(action.token);
            case ActionType.AuthSignInWithEmailAndPassword:
                return auth.signInWithEmailAndPassword(action.email, action.password);
            case ActionType.AuthSignInWithPhoneNumber:
                return auth.signInWithPhoneNumber(action.phoneNumber, action.verifier);
            case ActionType.AuthSignInWithPopup:
                return auth.signInWithPopup(action.provider);
            case ActionType.AuthSignInWithRedirect:
                return auth.signInWithRedirect(action.provider);
            case ActionType.AuthSignOut:
                return auth.signOut();
            case ActionType.AuthUseDeviceLanguage:
                return auth.useDeviceLanguage();
            case ActionType.AuthVerifyPasswordResetCode:
                return auth.verifyPasswordResetCode(action.code);
            case ActionType.DatabaseGoOffline:
                return db.goOffline();
            case ActionType.DatabaseGoOnline:
                return db.goOnline();
            case ActionType.ReferencePush:
                return new Promise(function (resolve) {
                    db
                        .ref(action.refPath)
                        .push(action.value)
                        .then(function (value) { return resolve(value); });
                });
            case ActionType.ReferenceRemove:
                return db.ref(action.refPath).remove();
            case ActionType.ReferenceSet:
                return db.ref(action.refPath).set(action.value);
            case ActionType.ReferenceSetPriority:
                return db.ref(action.refPath).setPriority(action.priority, function () { return null; });
            case ActionType.ReferenceSetWithPriority:
                return db
                    .ref(action.refPath)
                    .setWithPriority(action.value, action.priority);
            case ActionType.ReferenceTransaction:
                return db.ref(action.refPath).transaction(action.updateFn);
            case ActionType.ReferenceUpdate:
                return db.ref(action.refPath).update(action.values);
            case ActionType.UserUnlink:
                return action.user.unlink();
            case ActionType.UserUpdateEmail:
                return action.user.updateEmail(action.email);
            case ActionType.UserUpdatePhoneNumber:
                return action.user.updatePhoneNumber(action.phoneNumber);
            case ActionType.UserUpdateProfile:
                return action.user.updateProfile(action.profile);
            default:
                return new Promise(function (_resolve, reject) {
                    reject(new Error('No matching action type'));
                });
        }
    }
    return handleAction;
}
exports.makeActionHandler = makeActionHandler;
//# sourceMappingURL=actions.js.map