"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("firebase/auth");
require("firebase/database");
var xstream_1 = require("xstream");
var actions_1 = require("./actions");
var app_1 = require("firebase/app");
function makeFirebaseDriver(config, appName) {
    var app = app_1.initializeApp(config, appName);
    var auth = app.auth();
    var db = app.database();
    var handleAction = actions_1.makeActionHandler(app);
    function createResult$(action) {
        var result$ = xstream_1.Stream.createWithMemory({
            start: function (listener) {
                try {
                    var promise = handleAction(action);
                    promise
                        .catch(function (err) {
                        listener.error(err);
                    })
                        .then(function (result) {
                        listener.next(result);
                        listener.complete();
                    });
                }
                catch (err) {
                    listener.error(err);
                }
            },
            stop: function () { }
        });
        Object.defineProperty(result$, 'action', {
            value: action,
            writable: false
        });
        result$.addListener({
            complete: function () { },
            error: function () { },
            next: function () { }
        });
        return result$;
    }
    function firebaseDriver(action$) {
        var result$$ = action$.map(createResult$);
        result$$.addListener({
            complete: function () { },
            error: function () { },
            next: function () { }
        });
        var firebaseSource = {
            auth: {
                authState: xstream_1.Stream.createWithMemory({
                    start: function (listener) {
                        auth.onAuthStateChanged(function (user) {
                            listener.next(user);
                        }, function (err) {
                            listener.error(err);
                        }, function () {
                            listener.complete();
                        });
                    },
                    stop: function () { return null; }
                }),
                currentUser: xstream_1.Stream.createWithMemory({
                    start: function (listener) {
                        var currentUser = null;
                        auth.onIdTokenChanged(function (_user) {
                            if (auth.currentUser !== currentUser) {
                                currentUser = auth.currentUser;
                                listener.next(currentUser);
                            }
                        }, function (err) {
                            listener.error(err);
                        }, function () {
                            listener.complete();
                        });
                    },
                    stop: function () { return null; }
                }),
                idToken: xstream_1.Stream.createWithMemory({
                    start: function (listener) {
                        auth.onIdTokenChanged(function (user) {
                            listener.next(user);
                        }, function (err) {
                            listener.error(err);
                        }, function () {
                            listener.complete();
                        });
                    },
                    stop: function () { return null; }
                }),
                providersForEmail: function (email) {
                    return xstream_1.Stream.createWithMemory({
                        start: function (listener) {
                            auth
                                .fetchProvidersForEmail(email)
                                .catch(function (err) {
                                listener.error(err);
                            })
                                .then(function (providers) {
                                listener.next(providers);
                            });
                        },
                        stop: function () { return null; }
                    });
                },
                redirectResult: xstream_1.Stream.createWithMemory({
                    start: function (listener) {
                        auth
                            .getRedirectResult()
                            .catch(function (err) {
                            listener.error(err);
                        })
                            .then(function (result) {
                            listener.next(result);
                        });
                    },
                    stop: function () { return null; }
                })
            },
            database: {
                ref: function (path) { return sourceReference(db.ref(path)); },
                refFromURL: function (url) { return sourceReference(db.refFromURL(url)); }
            },
            select: function (category) {
                return typeof category === 'undefined'
                    ? result$$
                    : result$$.filter(function (result$) {
                        return result$.hasOwnProperty('action') &&
                            result$.action.category === category;
                    });
            }
        };
        return firebaseSource;
    }
    return firebaseDriver;
}
exports.makeFirebaseDriver = makeFirebaseDriver;
function makeReferenceEventsCallback(listener) {
    return function (snapshot) {
        if (snapshot !== null) {
            listener.next(snapshot.val());
        }
    };
}
function refEvents(ref) {
    return function (eventType) {
        var callback;
        return xstream_1.Stream.createWithMemory({
            start: function (listener) {
                callback = makeReferenceEventsCallback(listener);
                ref.on(eventType, callback);
            },
            stop: function () {
                ref.off(eventType, callback);
            }
        });
    };
}
function sourceReference(dbRef) {
    var events = refEvents(dbRef);
    return {
        child: function (path) { return sourceReference(dbRef.child(path)); },
        events: events,
        value: events('value')
    };
}
//# sourceMappingURL=driver.js.map